# -*- coding: utf-8 -*-

"""
DigitalPalette is a free software, which is distributed in the hope 
that it will be useful, but WITHOUT ANY WARRANTY. You can redistribute 
it and/or modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation. See the GNU General Public 
License for more details.

Please visit https://github.com/liujiacode/DigitalPalette for more 
infomation about VioletPy.

Copyright (c) 2019-2021 by Eigenmiao. All Rights Reserved.
"""

import os
import json
import locale
from PyQt5.QtWidgets import QSplashScreen
from PyQt5.QtGui import QPixmap, QImage
from cguis.resource import view_rc


class DPSplash(QSplashScreen):
    """
    Show image when loading.
    """

    def __init__(self, resources, sys_argv):
        """
        Init splash.
        """

        display_lang = "en"

        if sys_argv["lang"]:
            if sys_argv["lang"][:2].lower() in ("zh", "ja", "ko"):
                display_lang = "zh"

        else:
            try:
                default_locale = str(locale.getdefaultlocale()[0])

            except Exception as err:
                default_locale = ""

            if len(default_locale) > 1 and default_locale[:2].lower() in ("zh", "ja", "ko"):
                display_lang = "zh"

            else:
                try:
                    with open(os.sep.join((resources, "settings.json")), "r", encoding="utf-8") as sf:
                        uss = json.load(sf)

                    if isinstance(uss, dict) and "lang" in uss and str(uss["lang"])[:2].lower() in ("zh", "ja", "ko"):
                        display_lang = "zh"

                except Exception as err:
                    pass

        super().__init__()

        self.setPixmap(QPixmap.fromImage(QImage(":/images/images/repository_{}.png".format(display_lang))))

    def mousePressEvent(self, event):
        event.ignore()

    def mouseDoubleClickEvent(self, event):
        event.ignore()

    def enterEvent(self, event):
        event.ignore()

    def mouseMoveEvent(self, event):
        event.ignore()
